"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const index_1 = require("../../index");
const templating_1 = require("../../lib/templating");
const bundle_api_spec_1 = require("../bundle-api-spec");
const { render } = templating_1.createTemplateEnvironment({
    templateDir: `${templating_1.DEFAULT_TEMPLATE_DIR}/sdk`
});
/**
 * Generate models as well as package scaffolding for a sdk that talks to a provided api spec
 * @param options
 */
function generateSdk(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { inferAttr } = options, params = __rest(options, ["inferAttr"]);
        yield fs.ensureDir(params.outPath);
        const templateParams = inferAttr
            ? mergeParams(yield inferAttributesFromPackage(), params)
            : params;
        const renderedFiles = yield renderAll(listTemplates(), templateParams);
        yield writeAllGeneratedCodeFiles(options.outPath, renderedFiles);
        yield index_1.generateApi({
            camelCasedPropNames: options.camelCasedPropNames,
            defaultErrorType: options.defaultErrorType,
            defaultSuccessType: options.defaultSuccessType,
            definitionsDirPath: options.outPath,
            generateClient: true,
            specFilePath: options.specFilePath,
            strictInterfaces: options.strictInterfaces
        });
        yield bundle_api_spec_1.bundleApiSpec({
            outPath: `${options.outPath}/openapi.yaml`,
            specFilePath: options.specFilePath,
            version: options.version
        });
    });
}
exports.generateSdk = generateSdk;
function inferAttributesFromPackage() {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const pkg = yield fs
            .readFile(`${process.cwd()}/package.json`)
            .then(String)
            .then(JSON.parse)
            .catch(ex => {
            throw new Error(`Failed to read package.json from the current directory: ${ex}`);
        });
        return {
            name: `${pkg.name}-sdk`,
            version: pkg.version,
            description: `Generated SDK for ${pkg.name}. ${pkg.description}`,
            author: pkg.author,
            license: pkg.license,
            registry: (_a = pkg.publishConfig) === null || _a === void 0 ? void 0 : _a.registry,
            access: (_b = pkg.publishConfig) === null || _b === void 0 ? void 0 : _b.access
        };
    });
}
function mergeParams(a, b) {
    return Object.keys(Object.assign(Object.assign({}, a), b)).reduce((p, k) => (Object.assign(Object.assign({}, p), { [k]: b[k] || a[k] })), {});
}
function listTemplates() {
    return ["package.json.njk", "tsconfig.json.njk", "index.ts.njk"];
}
/**
 * Renders all templates and return a hashmap in the form (filepath, renderedCode)
 * @param options
 */
function renderAll(files, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const allContent = yield Promise.all(files.map(file => render(file, options)));
        return allContent.reduce((p, rendered, i) => (Object.assign(Object.assign({}, p), { [files[i]]: rendered })), {});
    });
}
exports.renderAll = renderAll;
/**
 * Wraps file writing to expose a common interface and log consistently
 * @param name name of the piece of code to render
 * @param outPath path of the file
 * @param code code to be saved
 *
 */
function writeGeneratedCodeFile(name, outPath, code) {
    console.log(`${name} -> ${outPath}`);
    return fs.writeFile(outPath, code);
}
function writeAllGeneratedCodeFiles(outPath, files) {
    return Promise.all(Object.keys(files).map((filepath) => writeGeneratedCodeFile(filepath, `${outPath}/${filepath}`.replace(".njk", ""), files[filepath])));
}
//# sourceMappingURL=index.js.map