"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const nunjucks = require("nunjucks");
const defaultFilters = require("./filters");
exports.DEFAULT_TEMPLATE_DIR = `${__dirname}/../../../templates`;
/**
 * Create an instance of teh template engine.
 * Default filters are included along side custom filters
 * @param templateDir base directory for templates
 * @param customFilters list of custom filters to apply to the environment
 */
exports.createTemplateEnvironment = ({ templateDir = exports.DEFAULT_TEMPLATE_DIR, customFilters = {} } = {}) => {
    nunjucks.configure({
        trimBlocks: true
    });
    const env = new nunjucks.Environment(new nunjucks.FileSystemLoader(templateDir));
    // make custom filters available in the rendered templates
    // tslint:disable-next-line: no-any
    const filters = Object.assign(Object.assign({}, defaultFilters), customFilters);
    Object.keys(filters).forEach(filterName => {
        const filter = filters[filterName];
        env.addFilter(filterName, filter);
    });
    /**
     * Override the default render function to return a Promise
     * @param templateName file name of the template to render
     * @param context optional object of data to pass to the template
     *
     * @return a promise of the rendered template
     */
    const render = (templateName, context) => new Promise((accept, reject) => {
        env.render(templateName, context, (err, res) => {
            if (err) {
                return reject(err);
            }
            accept(res || "");
        });
    });
    return Object.assign(Object.assign({}, env), { render });
};
exports.defaultTemplateEnvironment = exports.createTemplateEnvironment();
//# sourceMappingURL=index.js.map