"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Uppercase on first letter of a string
 * @param s string to be capitalized
 */
function capitalize(s) {
    return `${s[0].toUpperCase()}${s.slice(1)}`;
}
exports.capitalize = capitalize;
/**
 * Lowercase on first letter of a string
 * @param s string to be uncapitalized
 */
function uncapitalize(s) {
    return `${s[0].toLowerCase()}${s.slice(1)}`;
}
exports.uncapitalize = uncapitalize;
/**
 * Wrap a string in doublequote
 * @param s string to be wrapped
 */
exports.doubleQuote = (s) => `"${s}"`;
/**
 * Converts an array of terms into a string representing a union of literals
 * @param arr array of literals to be converted
 * @param onEmpty what to return in case of empty union
 */
exports.toUnionOfLiterals = (arr, onEmpty = "never") => arr.length ? arr.map(exports.doubleQuote).join(" | ") : onEmpty;
/**
 * Renders a type or variable name extended with its generics, if any
 * Examples:
 * ("MyType") -> "MyType"
 * ("MyType", []) -> "MyType"
 * ("MyType", ["T1", "T2"]) -> "MyType<T1, T2>"
 * @param name type or variable to name to render
 * @param generics list of generics
 *
 * @returns rendered name
 */
function withGenerics(name, generics = []) {
    return generics.length ? `${name}<${generics.join(", ")}>` : name;
}
exports.withGenerics = withGenerics;
exports.pipe = (...fns) => (value) => fns.reduce((p, f) => f(p), value);
//# sourceMappingURL=utils.js.map